@@ -26,6 +26,10 @@ SECTIONS = (TYPE, PROPERTIES, MEDADATA, DEPENDS_ON, UPDATE_POLICY,
             DELETION_POLICY) = \
            ('type', 'properties', 'metadata',
             'depends_on', 'update_policy', 'deletion_policy')
+
+policy_type = ['tosca.policies.Placement',
+               'tosca.policies.Scaling',
+               'tosca.policies.Scaling.Cluster']
 log = logging.getLogger('heat-translator')
 
 
@@ -46,7 +50,7 @@ class HotResource(object):
         # special case for HOT softwareconfig
         if type == 'OS::Heat::SoftwareConfig':
             config = self.properties.get('config')
-            if config:
+            if isinstance(config, dict):
                 implementation_artifact = config.get('get_file')
                 if implementation_artifact:
                     filename, file_extension = os.path.splitext(
@@ -104,7 +108,7 @@ class HotResource(object):
         # scenarios and cannot be fixed or hard coded here
         operations_deploy_sequence = ['create', 'configure', 'start']
 
-        operations = HotResource._get_all_operations(self.nodetemplate)
+        operations = HotResource.get_all_operations(self.nodetemplate)
 
         # create HotResource for each operation used for deployment:
         # create, start, configure
@@ -127,6 +131,14 @@ class HotResource(object):
         hosting_server = None
         if self.nodetemplate.requirements is not None:
             hosting_server = self._get_hosting_server()
+        # hosting_server is None if requirements is None
+        hosting_on_server = hosting_server.name if hosting_server else None
+        base_type = HotResource.get_base_type_str(
+            self.nodetemplate.type_definition)
+        # if we are on a compute node the host is self
+        if hosting_on_server is None and base_type == 'tosca.nodes.Compute':
+            hosting_on_server = self.name
+
         for operation in operations.values():
             if operation.name in operations_deploy_sequence:
                 config_name = node_name + '_' + operation.name + '_config'
@@ -137,18 +149,15 @@ class HotResource(object):
                                 'OS::Heat::SoftwareConfig',
                                 {'config':
                                     {'get_file': operation.implementation}}))
-
-                # hosting_server is None if requirements is None
-                hosting_on_server = (hosting_server.name if
-                                     hosting_server else None)
-                if operation.name == reserve_current:
+                if operation.name == reserve_current and \
+                    base_type != 'tosca.nodes.Compute':
                     deploy_resource = self
                     self.name = deploy_name
                     self.type = 'OS::Heat::SoftwareDeployment'
                     self.properties = {'config': {'get_resource': config_name},
                                        'server': {'get_resource':
                                                   hosting_on_server}}
-                    deploy_lookup[operation.name] = self
+                    deploy_lookup[operation] = self
                 else:
                     sd_config = {'config': {'get_resource': config_name},
                                  'server': {'get_resource':
@@ -159,7 +168,7 @@ class HotResource(object):
                                     'OS::Heat::SoftwareDeployment',
                                     sd_config)
                     hot_resources.append(deploy_resource)
-                    deploy_lookup[operation.name] = deploy_resource
+                    deploy_lookup[operation] = deploy_resource
                 lifecycle_inputs = self._get_lifecycle_inputs(operation)
                 if lifecycle_inputs:
                     deploy_resource.properties['input_values'] = \
@@ -169,24 +178,83 @@ class HotResource(object):
         # in operations_deploy_sequence
         # TODO(anyone): find some better way to encode this implicit sequence
         group = {}
+        op_index_min = None
+        op_index_max = -1
         for op, hot in deploy_lookup.items():
             # position to determine potential preceding nodes
-            op_index = operations_deploy_sequence.index(op)
-            for preceding_op in \
+            op_index = operations_deploy_sequence.index(op.name)
+            if op_index_min is None or op_index < op_index_min:
+                op_index_min = op_index
+            if op_index > op_index_max:
+                op_index_max = op_index
+            for preceding_op_name in \
                     reversed(operations_deploy_sequence[:op_index]):
-                preceding_hot = deploy_lookup.get(preceding_op)
+                preceding_hot = deploy_lookup.get(
+                    operations.get(preceding_op_name))
                 if preceding_hot:
                     hot.depends_on.append(preceding_hot)
                     hot.depends_on_nodes.append(preceding_hot)
                     group[preceding_hot] = hot
                     break
 
+        if op_index_max >= 0:
+            last_deploy = deploy_lookup.get(operations.get(
+                operations_deploy_sequence[op_index_max]))
+        else:
+            last_deploy = None
+
         # save this dependency chain in the set of HOT resources
         self.group_dependencies.update(group)
         for hot in hot_resources:
             hot.group_dependencies.update(group)
 
-        return hot_resources
+        roles_deploy_resource = self._handle_ansiblegalaxy_roles(
+            hot_resources, node_name, hosting_on_server)
+
+        # add a dependency to this ansible roles deploy to
+        # the first "classic" deploy generated for this node
+        if roles_deploy_resource and op_index_min:
+            first_deploy = deploy_lookup.get(operations.get(
+                operations_deploy_sequence[op_index_min]))
+            first_deploy.depends_on.append(roles_deploy_resource)
+            first_deploy.depends_on_nodes.append(roles_deploy_resource)
+
+        return hot_resources, deploy_lookup, last_deploy
+
+    def _handle_ansiblegalaxy_roles(self, hot_resources, initial_node_name,
+                                    hosting_on_server):
+        artifacts = self.get_all_artifacts(self.nodetemplate)
+        install_roles_script = ''
+        for artifact_name, artifact in six.iteritems(artifacts):
+            artifact_type = artifact.get('type', '').lower()
+            if artifact_type == 'tosca.artifacts.ansiblegalaxy.role':
+                role = artifact.get('file', None)
+                if role:
+                    install_roles_script += 'ansible-galaxy install ' + role \
+                                            + '\n'
+
+        if install_roles_script:
+            # remove trailing \n
+            install_roles_script = install_roles_script[:-1]
+            # add shebang and | to use literal scalar type (for multiline)
+            install_roles_script = '|\n#!/bin/bash\n' + install_roles_script
+
+            config_name = initial_node_name + '_install_roles_config'
+            deploy_name = initial_node_name + '_install_roles_deploy'
+            hot_resources.append(
+                HotResource(self.nodetemplate, config_name,
+                            'OS::Heat::SoftwareConfig',
+                            {'config': install_roles_script}))
+            sd_config = {'config': {'get_resource': config_name},
+                         'server': {'get_resource':
+                                    hosting_on_server}}
+            deploy_resource = \
+                HotResource(self.nodetemplate, deploy_name,
+                            'OS::Heat::SoftwareDeployment',
+                            sd_config)
+            hot_resources.append(deploy_resource)
+
+            return deploy_resource
 
     def handle_connectsto(self, tosca_source, tosca_target, hot_source,
                           hot_target, config_location, operation):
@@ -311,8 +379,8 @@ class HotResource(object):
         # capability is a list of dict
         # For now just check if it's type tosca.nodes.Compute
         # TODO(anyone): match up requirement and capability
-        base_type = HotResource.get_base_type(node.type_definition)
-        if base_type.type == 'tosca.nodes.Compute':
+        base_type = HotResource.get_base_type_str(node.type_definition)
+        if base_type == 'tosca.nodes.Compute':
             return True
         else:
             return False
@@ -333,15 +401,34 @@ class HotResource(object):
         return tosca_props
 
     @staticmethod
-    def _get_all_operations(node):
+    def get_all_artifacts(nodetemplate):
+        # workaround bug in the parser
+        base_type = HotResource.get_base_type_str(nodetemplate.type_definition)
+        if base_type in policy_type:
+            artifacts = {}
+        else:
+            artifacts = nodetemplate.type_definition.get_value('artifacts',
+                                                               parent=True)
+        if not artifacts:
+            artifacts = {}
+        tpl_artifacts = nodetemplate.entity_tpl.get('artifacts')
+        if tpl_artifacts:
+            artifacts.update(tpl_artifacts)
+
+        return artifacts
+
+    @staticmethod
+    def get_all_operations(node):
         operations = {}
         for operation in node.interfaces:
             operations[operation.name] = operation
 
+        # workaround bug in the parser
+        base_type = HotResource.get_base_type_str(node.type_definition)
+        if base_type in policy_type:
+            return operations
+
         node_type = node.type_definition
-        if isinstance(node_type, str) or \
-            node_type.type == "tosca.policies.Placement":
-                return operations
 
         while True:
             type_operations = HotResource._get_interface_operations_from_type(
@@ -357,9 +444,9 @@ class HotResource(object):
     @staticmethod
     def _get_interface_operations_from_type(node_type, node, lifecycle_name):
         operations = {}
-        if isinstance(node_type, str) or \
-            node_type.type == "tosca.policies.Placement":
-                return operations
+        base_type = HotResource.get_base_type_str(node_type)
+        if base_type in policy_type:
+            return operations
         if node_type.interfaces and lifecycle_name in node_type.interfaces:
             for name, elems in node_type.interfaces[lifecycle_name].items():
                 # ignore empty operations (only type)
@@ -372,11 +459,19 @@ class HotResource(object):
         return operations
 
     @staticmethod
-    def get_base_type(node_type):
-        if node_type.parent_type is not None:
-            if node_type.parent_type.type.endswith('.Root'):
-                return node_type
-            else:
-                return HotResource.get_base_type(node_type.parent_type)
-        else:
+    def get_base_type_str(node_type):
+        if isinstance(node_type, six.string_types):
             return node_type
+        if node_type.parent_type is not None:
+            parent_type_str = None
+            if isinstance(node_type.parent_type, six.string_types):
+                parent_type_str = node_type.parent_type
+            else:
+                parent_type_str = node_type.parent_type.type
+
+            if parent_type_str and parent_type_str.endswith('.Root'):
+                return node_type.type
+            else:
+                return HotResource.get_base_type_str(node_type.parent_type)
+
+        return node_type.type
