@@ -44,11 +44,11 @@ class HotTemplate(object):
             nodes.append((node_key, node_value))
         return yaml.nodes.MappingNode(u'tag:yaml.org,2002:map', nodes)
 
-    def output_to_yaml(self):
+    def output_to_yaml(self, hot_template_version=LATEST):
         log.debug(_('Converting translated output to yaml format.'))
         dict_output = OrderedDict()
         # Version
-        version_string = self.VERSION + ": " + self.LATEST + "\n\n"
+        version_string = self.VERSION + ": " + hot_template_version + "\n\n"
 
         # Description
         desc_str = ""
@@ -77,7 +77,10 @@ class HotTemplate(object):
         dict_output.update({self.OUTPUTS: all_outputs})
 
         yaml.add_representer(OrderedDict, self.represent_ordereddict)
+        yaml.add_representer(dict, self.represent_ordereddict)
         yaml_string = yaml.dump(dict_output, default_flow_style=False)
         # get rid of the '' from yaml.dump around numbers
-        yaml_string = yaml_string.replace('\'', '')
+        # also replace double return lines with a single one
+        # seems to be a bug in the serialization of multiline literal scalars
+        yaml_string = yaml_string.replace('\'', '') .replace('\n\n', '\n')
         return version_string + desc_str + yaml_string
